# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'HKLviewer.ui',
# licensing of 'HKLviewer.ui' applies.
#
# Created: Tue Mar  3 13:21:17 2020
#      by: pyside2-uic  running on PySide2 5.12.2
#
# WARNING! All changes made in this file will be lost!
from __future__ import absolute_import, division, print_function

from PySide2 import QtCore, QtWidgets

class Ui_MainWindow(object):
  def setupUi(self, MainWindow):
    MainWindow.setObjectName("MainWindow")
    MainWindow.resize(1066, 840)
    self.centralwidget = QtWidgets.QWidget(MainWindow)
    self.centralwidget.setObjectName("centralwidget")
    self.gridLayout_2 = QtWidgets.QGridLayout(self.centralwidget)
    self.gridLayout_2.setObjectName("gridLayout_2")
    self.widget = QtWidgets.QWidget(self.centralwidget)
    self.widget.setObjectName("widget")
    self.gridLayout = QtWidgets.QGridLayout(self.widget)
    self.gridLayout.setContentsMargins(0, 0, 0, 0)
    self.gridLayout.setObjectName("gridLayout")
    self.splitter = QtWidgets.QSplitter(self.widget)
    self.splitter.setOrientation(QtCore.Qt.Horizontal)
    self.splitter.setObjectName("splitter")
    self.widget_2 = QtWidgets.QWidget(self.splitter)
    self.widget_2.setObjectName("widget_2")
    self.gridLayout_4 = QtWidgets.QGridLayout(self.widget_2)
    self.gridLayout_4.setContentsMargins(0, 0, 0, 0)
    self.gridLayout_4.setObjectName("gridLayout_4")
    self.splitter_2 = QtWidgets.QSplitter(self.widget_2)
    self.splitter_2.setFrameShape(QtWidgets.QFrame.NoFrame)
    self.splitter_2.setFrameShadow(QtWidgets.QFrame.Plain)
    self.splitter_2.setMidLineWidth(0)
    self.splitter_2.setOrientation(QtCore.Qt.Vertical)
    self.splitter_2.setObjectName("splitter_2")
    self.widget_4 = QtWidgets.QWidget(self.splitter_2)
    sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
    sizePolicy.setHorizontalStretch(0)
    sizePolicy.setVerticalStretch(3)
    sizePolicy.setHeightForWidth(self.widget_4.sizePolicy().hasHeightForWidth())
    self.widget_4.setSizePolicy(sizePolicy)
    self.widget_4.setObjectName("widget_4")
    self.gridLayout_5 = QtWidgets.QGridLayout(self.widget_4)
    self.gridLayout_5.setContentsMargins(0, 6, 0, -1)
    self.gridLayout_5.setObjectName("gridLayout_5")
    self.label = QtWidgets.QLabel(self.widget_4)
    self.label.setObjectName("label")
    self.gridLayout_5.addWidget(self.label, 0, 0, 1, 1)
    self.millertable = HeaderDataTableWidget(self.widget_4)
    sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
    sizePolicy.setHorizontalStretch(0)
    sizePolicy.setVerticalStretch(0)
    sizePolicy.setHeightForWidth(self.millertable.sizePolicy().hasHeightForWidth())
    self.millertable.setSizePolicy(sizePolicy)
    self.millertable.setRowCount(5)
    self.millertable.setColumnCount(10)
    self.millertable.setObjectName("millertable")
    self.millertable.setColumnCount(10)
    self.millertable.setRowCount(5)
    self.gridLayout_5.addWidget(self.millertable, 1, 0, 1, 1)
    self.functionTabWidget = QtWidgets.QTabWidget(self.splitter_2)
    sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Preferred)
    sizePolicy.setHorizontalStretch(0)
    sizePolicy.setVerticalStretch(1)
    sizePolicy.setHeightForWidth(self.functionTabWidget.sizePolicy().hasHeightForWidth())
    self.functionTabWidget.setSizePolicy(sizePolicy)
    self.functionTabWidget.setObjectName("functionTabWidget")
    self.tab = QtWidgets.QWidget()
    self.tab.setObjectName("tab")
    self.gridLayout_28 = QtWidgets.QGridLayout(self.tab)
    self.gridLayout_28.setObjectName("gridLayout_28")
    self.ExpandReflsGroupBox = QtWidgets.QGroupBox(self.tab)
    sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
    sizePolicy.setHorizontalStretch(0)
    sizePolicy.setVerticalStretch(0)
    sizePolicy.setHeightForWidth(self.ExpandReflsGroupBox.sizePolicy().hasHeightForWidth())
    self.ExpandReflsGroupBox.setSizePolicy(sizePolicy)
    self.ExpandReflsGroupBox.setCheckable(True)
    self.ExpandReflsGroupBox.setChecked(False)
    self.ExpandReflsGroupBox.setObjectName("ExpandReflsGroupBox")
    self.gridLayout_3 = QtWidgets.QGridLayout(self.ExpandReflsGroupBox)
    self.gridLayout_3.setContentsMargins(6, 6, 6, 6)
    self.gridLayout_3.setObjectName("gridLayout_3")
    self.expandP1checkbox = QtWidgets.QCheckBox(self.ExpandReflsGroupBox)
    self.expandP1checkbox.setObjectName("expandP1checkbox")
    self.gridLayout_3.addWidget(self.expandP1checkbox, 0, 0, 1, 1)
    self.expandAnomalouscheckbox = QtWidgets.QCheckBox(self.ExpandReflsGroupBox)
    self.expandAnomalouscheckbox.setObjectName("expandAnomalouscheckbox")
    self.gridLayout_3.addWidget(self.expandAnomalouscheckbox, 0, 1, 1, 1)
    self.gridLayout_28.addWidget(self.ExpandReflsGroupBox, 0, 0, 1, 3)
    self.SpacegroupLabel = QtWidgets.QLabel(self.tab)
    sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
    sizePolicy.setHorizontalStretch(0)
    sizePolicy.setVerticalStretch(2)
    sizePolicy.setHeightForWidth(self.SpacegroupLabel.sizePolicy().hasHeightForWidth())
    self.SpacegroupLabel.setSizePolicy(sizePolicy)
    self.SpacegroupLabel.setObjectName("SpacegroupLabel")
    self.gridLayout_28.addWidget(self.SpacegroupLabel, 2, 0, 1, 1)
    self.sysabsentcheckbox = QtWidgets.QCheckBox(self.tab)
    self.sysabsentcheckbox.setObjectName("sysabsentcheckbox")
    self.gridLayout_28.addWidget(self.sysabsentcheckbox, 1, 0, 1, 1)
    self.widget_3 = QtWidgets.QWidget(self.tab)
    self.widget_3.setObjectName("widget_3")
    self.gridLayout_20 = QtWidgets.QGridLayout(self.widget_3)
    self.gridLayout_20.setContentsMargins(0, 0, 0, 0)
    self.gridLayout_20.setObjectName("gridLayout_20")
    self.missingcheckbox = QtWidgets.QCheckBox(self.widget_3)
    self.missingcheckbox.setObjectName("missingcheckbox")
    self.gridLayout_20.addWidget(self.missingcheckbox, 0, 0, 2, 2)
    self.onlymissingcheckbox = QtWidgets.QCheckBox(self.widget_3)
    self.onlymissingcheckbox.setObjectName("onlymissingcheckbox")
    self.gridLayout_20.addWidget(self.onlymissingcheckbox, 2, 0, 1, 1)
    self.gridLayout_28.addWidget(self.widget_3, 1, 1, 1, 2)
    self.SpaceGroupComboBox = QtWidgets.QComboBox(self.tab)
    sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
    sizePolicy.setHorizontalStretch(0)
    sizePolicy.setVerticalStretch(2)
    sizePolicy.setHeightForWidth(self.SpaceGroupComboBox.sizePolicy().hasHeightForWidth())
    self.SpaceGroupComboBox.setSizePolicy(sizePolicy)
    self.SpaceGroupComboBox.setObjectName("SpaceGroupComboBox")
    self.gridLayout_28.addWidget(self.SpaceGroupComboBox, 2, 1, 1, 2)
    self.functionTabWidget.addTab(self.tab, "")
    self.tab_2 = QtWidgets.QWidget()
    self.tab_2.setObjectName("tab_2")
    self.gridLayout_9 = QtWidgets.QGridLayout(self.tab_2)
    self.gridLayout_9.setObjectName("gridLayout_9")
    self.SliceReflectionsBox = QtWidgets.QCheckBox(self.tab_2)
    sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Fixed)
    sizePolicy.setHorizontalStretch(1)
    sizePolicy.setVerticalStretch(10)
    sizePolicy.setHeightForWidth(self.SliceReflectionsBox.sizePolicy().hasHeightForWidth())
    self.SliceReflectionsBox.setSizePolicy(sizePolicy)
    self.SliceReflectionsBox.setObjectName("SliceReflectionsBox")
    self.gridLayout_9.addWidget(self.SliceReflectionsBox, 0, 0, 1, 1)
    self.fixedorientcheckbox = QtWidgets.QCheckBox(self.tab_2)
    sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Fixed)
    sizePolicy.setHorizontalStretch(3)
    sizePolicy.setVerticalStretch(0)
    sizePolicy.setHeightForWidth(self.fixedorientcheckbox.sizePolicy().hasHeightForWidth())
    self.fixedorientcheckbox.setSizePolicy(sizePolicy)
    self.fixedorientcheckbox.setObjectName("fixedorientcheckbox")
    self.gridLayout_9.addWidget(self.fixedorientcheckbox, 0, 1, 1, 1)
    self.ClipPlaneChkGroupBox = QtWidgets.QGroupBox(self.tab_2)
    sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
    sizePolicy.setHorizontalStretch(0)
    sizePolicy.setVerticalStretch(1)
    sizePolicy.setHeightForWidth(self.ClipPlaneChkGroupBox.sizePolicy().hasHeightForWidth())
    self.ClipPlaneChkGroupBox.setSizePolicy(sizePolicy)
    self.ClipPlaneChkGroupBox.setCheckable(True)
    self.ClipPlaneChkGroupBox.setObjectName("ClipPlaneChkGroupBox")
    self.gridLayout_7 = QtWidgets.QGridLayout(self.ClipPlaneChkGroupBox)
    self.gridLayout_7.setContentsMargins(3, 3, 3, 6)
    self.gridLayout_7.setObjectName("gridLayout_7")
    self.clipNormalBtn = QtWidgets.QRadioButton(self.ClipPlaneChkGroupBox)
    sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
    sizePolicy.setHorizontalStretch(0)
    sizePolicy.setVerticalStretch(0)
    sizePolicy.setHeightForWidth(self.clipNormalBtn.sizePolicy().hasHeightForWidth())
    self.clipNormalBtn.setSizePolicy(sizePolicy)
    self.clipNormalBtn.setObjectName("clipNormalBtn")
    self.gridLayout_7.addWidget(self.clipNormalBtn, 0, 1, 1, 1)
    self.clipParallelBtn = QtWidgets.QRadioButton(self.ClipPlaneChkGroupBox)
    sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
    sizePolicy.setHorizontalStretch(0)
    sizePolicy.setVerticalStretch(0)
    sizePolicy.setHeightForWidth(self.clipParallelBtn.sizePolicy().hasHeightForWidth())
    self.clipParallelBtn.setSizePolicy(sizePolicy)
    self.clipParallelBtn.setObjectName("clipParallelBtn")
    self.gridLayout_7.addWidget(self.clipParallelBtn, 0, 0, 1, 1)
    self.groupBox_2 = QtWidgets.QGroupBox(self.ClipPlaneChkGroupBox)
    self.groupBox_2.setObjectName("groupBox_2")
    self.gridLayout_19 = QtWidgets.QGridLayout(self.groupBox_2)
    self.gridLayout_19.setObjectName("gridLayout_19")
    self.frame_2 = QtWidgets.QFrame(self.groupBox_2)
    sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
    sizePolicy.setHorizontalStretch(0)
    sizePolicy.setVerticalStretch(0)
    sizePolicy.setHeightForWidth(self.frame_2.sizePolicy().hasHeightForWidth())
    self.frame_2.setSizePolicy(sizePolicy)
    self.frame_2.setFrameShape(QtWidgets.QFrame.StyledPanel)
    self.frame_2.setFrameShadow(QtWidgets.QFrame.Raised)
    self.frame_2.setObjectName("frame_2")
    self.gridLayout_15 = QtWidgets.QGridLayout(self.frame_2)
    self.gridLayout_15.setContentsMargins(0, 0, 0, 0)
    self.gridLayout_15.setObjectName("gridLayout_15")
    self.recipvecBtn = QtWidgets.QRadioButton(self.frame_2)
    self.recipvecBtn.setObjectName("recipvecBtn")
    self.gridLayout_15.addWidget(self.recipvecBtn, 0, 0, 1, 1)
    self.clipTNCSBtn = QtWidgets.QRadioButton(self.frame_2)
    self.clipTNCSBtn.setObjectName("clipTNCSBtn")
    self.gridLayout_15.addWidget(self.clipTNCSBtn, 0, 1, 1, 1)
    self.realspacevecBtn = QtWidgets.QRadioButton(self.frame_2)
    self.realspacevecBtn.setObjectName("realspacevecBtn")
    self.gridLayout_15.addWidget(self.realspacevecBtn, 1, 0, 1, 1)
    self.gridLayout_19.addWidget(self.frame_2, 0, 0, 1, 1)
    self.frame_6 = QtWidgets.QFrame(self.groupBox_2)
    self.frame_6.setFrameShape(QtWidgets.QFrame.StyledPanel)
    self.frame_6.setFrameShadow(QtWidgets.QFrame.Raised)
    self.frame_6.setObjectName("frame_6")
    self.gridLayout_18 = QtWidgets.QGridLayout(self.frame_6)
    self.gridLayout_18.setContentsMargins(0, 0, 0, 0)
    self.gridLayout_18.setObjectName("gridLayout_18")
    self.rotavecangle_labeltxt = QtWidgets.QLabel(self.frame_6)
    sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
    sizePolicy.setHorizontalStretch(0)
    sizePolicy.setVerticalStretch(0)
    sizePolicy.setHeightForWidth(self.rotavecangle_labeltxt.sizePolicy().hasHeightForWidth())
    self.rotavecangle_labeltxt.setSizePolicy(sizePolicy)
    self.rotavecangle_labeltxt.setObjectName("rotavecangle_labeltxt")
    self.gridLayout_18.addWidget(self.rotavecangle_labeltxt, 0, 0, 1, 1)
    self.rotavecangle_slider = QtWidgets.QSlider(self.frame_6)
    sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
    sizePolicy.setHorizontalStretch(1)
    sizePolicy.setVerticalStretch(0)
    sizePolicy.setHeightForWidth(self.rotavecangle_slider.sizePolicy().hasHeightForWidth())
    self.rotavecangle_slider.setSizePolicy(sizePolicy)
    self.rotavecangle_slider.setMaximum(359)
    self.rotavecangle_slider.setOrientation(QtCore.Qt.Horizontal)
    self.rotavecangle_slider.setTickPosition(QtWidgets.QSlider.TicksAbove)
    self.rotavecangle_slider.setTickInterval(18)
    self.rotavecangle_slider.setObjectName("rotavecangle_slider")
    self.gridLayout_18.addWidget(self.rotavecangle_slider, 0, 1, 1, 1)
    self.gridLayout_19.addWidget(self.frame_6, 3, 0, 1, 1)
    self.frame_3 = QtWidgets.QFrame(self.groupBox_2)
    self.frame_3.setFrameShape(QtWidgets.QFrame.StyledPanel)
    self.frame_3.setFrameShadow(QtWidgets.QFrame.Raised)
    self.frame_3.setObjectName("frame_3")
    self.gridLayout_16 = QtWidgets.QGridLayout(self.frame_3)
    self.gridLayout_16.setContentsMargins(0, 0, 0, 0)
    self.gridLayout_16.setObjectName("gridLayout_16")
    self.frame_9 = QtWidgets.QFrame(self.frame_3)
    sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Preferred)
    sizePolicy.setHorizontalStretch(0)
    sizePolicy.setVerticalStretch(0)
    sizePolicy.setHeightForWidth(self.frame_9.sizePolicy().hasHeightForWidth())
    self.frame_9.setSizePolicy(sizePolicy)
    self.frame_9.setFrameShape(QtWidgets.QFrame.StyledPanel)
    self.frame_9.setFrameShadow(QtWidgets.QFrame.Raised)
    self.frame_9.setObjectName("frame_9")
    self.gridLayout_22 = QtWidgets.QGridLayout(self.frame_9)
    self.gridLayout_22.setContentsMargins(0, 0, 0, 0)
    self.gridLayout_22.setObjectName("gridLayout_22")
    self.label_21 = QtWidgets.QLabel(self.frame_9)
    sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Preferred)
    sizePolicy.setHorizontalStretch(0)
    sizePolicy.setVerticalStretch(0)
    sizePolicy.setHeightForWidth(self.label_21.sizePolicy().hasHeightForWidth())
    self.label_21.setSizePolicy(sizePolicy)
    self.label_21.setObjectName("label_21")
    self.gridLayout_22.addWidget(self.label_21, 0, 0, 1, 1)
    self.hvec_spinBox = QtWidgets.QDoubleSpinBox(self.frame_9)
    sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Preferred)
    sizePolicy.setHorizontalStretch(0)
    sizePolicy.setVerticalStretch(0)
    sizePolicy.setHeightForWidth(self.hvec_spinBox.sizePolicy().hasHeightForWidth())
    self.hvec_spinBox.setSizePolicy(sizePolicy)
    self.hvec_spinBox.setObjectName("hvec_spinBox")
    self.gridLayout_22.addWidget(self.hvec_spinBox, 0, 1, 1, 1)
    self.gridLayout_16.addWidget(self.frame_9, 0, 0, 1, 1)
    self.frame_10 = QtWidgets.QFrame(self.frame_3)
    sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Preferred)
    sizePolicy.setHorizontalStretch(0)
    sizePolicy.setVerticalStretch(0)
    sizePolicy.setHeightForWidth(self.frame_10.sizePolicy().hasHeightForWidth())
    self.frame_10.setSizePolicy(sizePolicy)
    self.frame_10.setFrameShape(QtWidgets.QFrame.StyledPanel)
    self.frame_10.setFrameShadow(QtWidgets.QFrame.Raised)
    self.frame_10.setObjectName("frame_10")
    self.gridLayout_23 = QtWidgets.QGridLayout(self.frame_10)
    self.gridLayout_23.setContentsMargins(0, 0, 0, 0)
    self.gridLayout_23.setObjectName("gridLayout_23")
    self.label_22 = QtWidgets.QLabel(self.frame_10)
    sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Preferred)
    sizePolicy.setHorizontalStretch(0)
    sizePolicy.setVerticalStretch(0)
    sizePolicy.setHeightForWidth(self.label_22.sizePolicy().hasHeightForWidth())
    self.label_22.setSizePolicy(sizePolicy)
    self.label_22.setObjectName("label_22")
    self.gridLayout_23.addWidget(self.label_22, 0, 0, 1, 1)
    self.kvec_spinBox = QtWidgets.QDoubleSpinBox(self.frame_10)
    sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Preferred)
    sizePolicy.setHorizontalStretch(0)
    sizePolicy.setVerticalStretch(0)
    sizePolicy.setHeightForWidth(self.kvec_spinBox.sizePolicy().hasHeightForWidth())
    self.kvec_spinBox.setSizePolicy(sizePolicy)
    self.kvec_spinBox.setObjectName("kvec_spinBox")
    self.gridLayout_23.addWidget(self.kvec_spinBox, 0, 1, 1, 1)
    self.gridLayout_16.addWidget(self.frame_10, 0, 1, 1, 1)
    self.frame_11 = QtWidgets.QFrame(self.frame_3)
    sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Preferred)
    sizePolicy.setHorizontalStretch(0)
    sizePolicy.setVerticalStretch(0)
    sizePolicy.setHeightForWidth(self.frame_11.sizePolicy().hasHeightForWidth())
    self.frame_11.setSizePolicy(sizePolicy)
    self.frame_11.setFrameShape(QtWidgets.QFrame.StyledPanel)
    self.frame_11.setFrameShadow(QtWidgets.QFrame.Raised)
    self.frame_11.setObjectName("frame_11")
    self.gridLayout_24 = QtWidgets.QGridLayout(self.frame_11)
    self.gridLayout_24.setContentsMargins(0, 0, 0, 0)
    self.gridLayout_24.setObjectName("gridLayout_24")
    self.label_23 = QtWidgets.QLabel(self.frame_11)
    sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Preferred)
    sizePolicy.setHorizontalStretch(0)
    sizePolicy.setVerticalStretch(0)
    sizePolicy.setHeightForWidth(self.label_23.sizePolicy().hasHeightForWidth())
    self.label_23.setSizePolicy(sizePolicy)
    self.label_23.setObjectName("label_23")
    self.gridLayout_24.addWidget(self.label_23, 0, 0, 1, 1)
    self.lvec_spinBox = QtWidgets.QDoubleSpinBox(self.frame_11)
    sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Preferred)
    sizePolicy.setHorizontalStretch(0)
    sizePolicy.setVerticalStretch(0)
    sizePolicy.setHeightForWidth(self.lvec_spinBox.sizePolicy().hasHeightForWidth())
    self.lvec_spinBox.setSizePolicy(sizePolicy)
    self.lvec_spinBox.setObjectName("lvec_spinBox")
    self.gridLayout_24.addWidget(self.lvec_spinBox, 0, 1, 1, 1)
    self.gridLayout_16.addWidget(self.frame_11, 0, 2, 1, 1)
    self.gridLayout_19.addWidget(self.frame_3, 1, 0, 1, 1)
    self.frame = QtWidgets.QFrame(self.groupBox_2)
    self.frame.setFrameShape(QtWidgets.QFrame.StyledPanel)
    self.frame.setFrameShadow(QtWidgets.QFrame.Raised)
    self.frame.setObjectName("frame")
    self.gridLayout_29 = QtWidgets.QGridLayout(self.frame)
    self.gridLayout_29.setContentsMargins(0, 0, 0, 0)
    self.gridLayout_29.setObjectName("gridLayout_29")
    self.label_15 = QtWidgets.QLabel(self.frame)
    sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Preferred)
    sizePolicy.setHorizontalStretch(0)
    sizePolicy.setVerticalStretch(0)
    sizePolicy.setHeightForWidth(self.label_15.sizePolicy().hasHeightForWidth())
    self.label_15.setSizePolicy(sizePolicy)
    self.label_15.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
    self.label_15.setObjectName("label_15")
    self.gridLayout_29.addWidget(self.label_15, 0, 3, 1, 1)
    self.hkldist_spinBox = QtWidgets.QDoubleSpinBox(self.frame)
    sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Preferred)
    sizePolicy.setHorizontalStretch(0)
    sizePolicy.setVerticalStretch(0)
    sizePolicy.setHeightForWidth(self.hkldist_spinBox.sizePolicy().hasHeightForWidth())
    self.hkldist_spinBox.setSizePolicy(sizePolicy)
    self.hkldist_spinBox.setObjectName("hkldist_spinBox")
    self.gridLayout_29.addWidget(self.hkldist_spinBox, 0, 1, 1, 1)
    self.clipwidth_spinBox = QtWidgets.QDoubleSpinBox(self.frame)
    sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Preferred)
    sizePolicy.setHorizontalStretch(0)
    sizePolicy.setVerticalStretch(0)
    sizePolicy.setHeightForWidth(self.clipwidth_spinBox.sizePolicy().hasHeightForWidth())
    self.clipwidth_spinBox.setSizePolicy(sizePolicy)
    self.clipwidth_spinBox.setObjectName("clipwidth_spinBox")
    self.gridLayout_29.addWidget(self.clipwidth_spinBox, 0, 4, 1, 1)
    self.label_14 = QtWidgets.QLabel(self.frame)
    self.label_14.setObjectName("label_14")
    self.gridLayout_29.addWidget(self.label_14, 0, 0, 1, 1)
    self.gridLayout_19.addWidget(self.frame, 2, 0, 1, 1)
    self.gridLayout_7.addWidget(self.groupBox_2, 1, 0, 1, 3)
    self.gridLayout_9.addWidget(self.ClipPlaneChkGroupBox, 1, 0, 1, 2)
    self.showsliceGroupCheckbox = QtWidgets.QGroupBox(self.tab_2)
    sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Minimum)
    sizePolicy.setHorizontalStretch(0)
    sizePolicy.setVerticalStretch(0)
    sizePolicy.setHeightForWidth(self.showsliceGroupCheckbox.sizePolicy().hasHeightForWidth())
    self.showsliceGroupCheckbox.setSizePolicy(sizePolicy)
    self.showsliceGroupCheckbox.setCheckable(True)
    self.showsliceGroupCheckbox.setChecked(True)
    self.showsliceGroupCheckbox.setObjectName("showsliceGroupCheckbox")
    self.gridLayout_6 = QtWidgets.QGridLayout(self.showsliceGroupCheckbox)
    self.gridLayout_6.setContentsMargins(3, 3, 3, 3)
    self.gridLayout_6.setObjectName("gridLayout_6")
    self.SliceLabelComboBox = QtWidgets.QComboBox(self.showsliceGroupCheckbox)
    sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
    sizePolicy.setHorizontalStretch(0)
    sizePolicy.setVerticalStretch(0)
    sizePolicy.setHeightForWidth(self.SliceLabelComboBox.sizePolicy().hasHeightForWidth())
    self.SliceLabelComboBox.setSizePolicy(sizePolicy)
    self.SliceLabelComboBox.setObjectName("SliceLabelComboBox")
    self.gridLayout_6.addWidget(self.SliceLabelComboBox, 0, 0, 1, 1)
    self.label_3 = QtWidgets.QLabel(self.showsliceGroupCheckbox)
    sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
    sizePolicy.setHorizontalStretch(0)
    sizePolicy.setVerticalStretch(0)
    sizePolicy.setHeightForWidth(self.label_3.sizePolicy().hasHeightForWidth())
    self.label_3.setSizePolicy(sizePolicy)
    self.label_3.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
    self.label_3.setIndent(1)
    self.label_3.setObjectName("label_3")
    self.gridLayout_6.addWidget(self.label_3, 0, 1, 1, 1)
    self.sliceindexspinBox = QtWidgets.QSpinBox(self.showsliceGroupCheckbox)
    sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
    sizePolicy.setHorizontalStretch(0)
    sizePolicy.setVerticalStretch(0)
    sizePolicy.setHeightForWidth(self.sliceindexspinBox.sizePolicy().hasHeightForWidth())
    self.sliceindexspinBox.setSizePolicy(sizePolicy)
    self.sliceindexspinBox.setObjectName("sliceindexspinBox")
    self.gridLayout_6.addWidget(self.sliceindexspinBox, 0, 2, 1, 1)
    self.gridLayout_9.addWidget(self.showsliceGroupCheckbox, 2, 0, 1, 2)
    self.functionTabWidget.addTab(self.tab_2, "")
    self.tab_3 = QtWidgets.QWidget()
    self.tab_3.setObjectName("tab_3")
    self.gridLayout_27 = QtWidgets.QGridLayout(self.tab_3)
    self.gridLayout_27.setObjectName("gridLayout_27")
    self.groupBox_3 = QtWidgets.QGroupBox(self.tab_3)
    sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
    sizePolicy.setHorizontalStretch(1)
    sizePolicy.setVerticalStretch(0)
    sizePolicy.setHeightForWidth(self.groupBox_3.sizePolicy().hasHeightForWidth())
    self.groupBox_3.setSizePolicy(sizePolicy)
    self.groupBox_3.setCheckable(False)
    self.groupBox_3.setObjectName("groupBox_3")
    self.gridLayout_10 = QtWidgets.QGridLayout(self.groupBox_3)
    self.gridLayout_10.setObjectName("gridLayout_10")
    self.label_7 = QtWidgets.QLabel(self.groupBox_3)
    sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
    sizePolicy.setHorizontalStretch(0)
    sizePolicy.setVerticalStretch(0)
    sizePolicy.setHeightForWidth(self.label_7.sizePolicy().hasHeightForWidth())
    self.label_7.setSizePolicy(sizePolicy)
    self.label_7.setTextFormat(QtCore.Qt.AutoText)
    self.label_7.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
    self.label_7.setWordWrap(True)
    self.label_7.setObjectName("label_7")
    self.gridLayout_10.addWidget(self.label_7, 0, 0, 1, 4)
    self.ManualPowerScalecheckbox = QtWidgets.QCheckBox(self.groupBox_3)
    sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.MinimumExpanding)
    sizePolicy.setHorizontalStretch(0)
    sizePolicy.setVerticalStretch(0)
    sizePolicy.setHeightForWidth(self.ManualPowerScalecheckbox.sizePolicy().hasHeightForWidth())
    self.ManualPowerScalecheckbox.setSizePolicy(sizePolicy)
    self.ManualPowerScalecheckbox.setToolTip("")
    self.ManualPowerScalecheckbox.setObjectName("ManualPowerScalecheckbox")
    self.gridLayout_10.addWidget(self.ManualPowerScalecheckbox, 1, 0, 1, 2)
    self.label_10 = QtWidgets.QLabel(self.groupBox_3)
    sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.MinimumExpanding)
    sizePolicy.setHorizontalStretch(0)
    sizePolicy.setVerticalStretch(0)
    sizePolicy.setHeightForWidth(self.label_10.sizePolicy().hasHeightForWidth())
    self.label_10.setSizePolicy(sizePolicy)
    self.label_10.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
    self.label_10.setIndent(1)
    self.label_10.setObjectName("label_10")
    self.gridLayout_10.addWidget(self.label_10, 1, 2, 1, 1)
    self.power_scale_spinBox = QtWidgets.QDoubleSpinBox(self.groupBox_3)
    sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Preferred)
    sizePolicy.setHorizontalStretch(0)
    sizePolicy.setVerticalStretch(0)
    sizePolicy.setHeightForWidth(self.power_scale_spinBox.sizePolicy().hasHeightForWidth())
    self.power_scale_spinBox.setSizePolicy(sizePolicy)
    self.power_scale_spinBox.setToolTip("")
    self.power_scale_spinBox.setMaximum(1.0)
    self.power_scale_spinBox.setSingleStep(0.05)
    self.power_scale_spinBox.setProperty("value", 0.35)
    self.power_scale_spinBox.setObjectName("power_scale_spinBox")
    self.gridLayout_10.addWidget(self.power_scale_spinBox, 1, 3, 1, 1)
    self.label_11 = QtWidgets.QLabel(self.groupBox_3)
    sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.MinimumExpanding)
    sizePolicy.setHorizontalStretch(0)
    sizePolicy.setVerticalStretch(0)
    sizePolicy.setHeightForWidth(self.label_11.sizePolicy().hasHeightForWidth())
    self.label_11.setSizePolicy(sizePolicy)
    self.label_11.setObjectName("label_11")
    self.gridLayout_10.addWidget(self.label_11, 2, 0, 1, 1)
    self.radii_scale_spinBox = QtWidgets.QDoubleSpinBox(self.groupBox_3)
    sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Preferred)
    sizePolicy.setHorizontalStretch(0)
    sizePolicy.setVerticalStretch(0)
    sizePolicy.setHeightForWidth(self.radii_scale_spinBox.sizePolicy().hasHeightForWidth())
    self.radii_scale_spinBox.setSizePolicy(sizePolicy)
    self.radii_scale_spinBox.setMaximum(5.0)
    self.radii_scale_spinBox.setSingleStep(0.1)
    self.radii_scale_spinBox.setObjectName("radii_scale_spinBox")
    self.gridLayout_10.addWidget(self.radii_scale_spinBox, 2, 1, 1, 1)
    self.gridLayout_27.addWidget(self.groupBox_3, 0, 0, 1, 1)
    self.functionTabWidget.addTab(self.tab_3, "")
    self.tab_4 = QtWidgets.QWidget()
    self.tab_4.setObjectName("tab_4")
    self.gridLayout_13 = QtWidgets.QGridLayout(self.tab_4)
    self.gridLayout_13.setObjectName("gridLayout_13")
    self.groupBox_5 = QtWidgets.QGroupBox(self.tab_4)
    sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
    sizePolicy.setHorizontalStretch(0)
    sizePolicy.setVerticalStretch(0)
    sizePolicy.setHeightForWidth(self.groupBox_5.sizePolicy().hasHeightForWidth())
    self.groupBox_5.setSizePolicy(sizePolicy)
    self.groupBox_5.setObjectName("groupBox_5")
    self.gridLayout_11 = QtWidgets.QGridLayout(self.groupBox_5)
    self.gridLayout_11.setObjectName("gridLayout_11")
    self.BinDataComboBox = QtWidgets.QComboBox(self.groupBox_5)
    sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
    sizePolicy.setHorizontalStretch(2)
    sizePolicy.setVerticalStretch(0)
    sizePolicy.setHeightForWidth(self.BinDataComboBox.sizePolicy().hasHeightForWidth())
    self.BinDataComboBox.setSizePolicy(sizePolicy)
    self.BinDataComboBox.setObjectName("BinDataComboBox")
    self.gridLayout_11.addWidget(self.BinDataComboBox, 0, 0, 1, 1)
    self.label_12 = QtWidgets.QLabel(self.groupBox_5)
    self.label_12.setObjectName("label_12")
    self.gridLayout_11.addWidget(self.label_12, 0, 1, 1, 1)
    self.Nbins_spinBox = QtWidgets.QSpinBox(self.groupBox_5)
    sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Preferred)
    sizePolicy.setHorizontalStretch(0)
    sizePolicy.setVerticalStretch(0)
    sizePolicy.setHeightForWidth(self.Nbins_spinBox.sizePolicy().hasHeightForWidth())
    self.Nbins_spinBox.setSizePolicy(sizePolicy)
    self.Nbins_spinBox.setMinimum(1)
    self.Nbins_spinBox.setMaximum(20)
    self.Nbins_spinBox.setObjectName("Nbins_spinBox")
    self.gridLayout_11.addWidget(self.Nbins_spinBox, 0, 2, 1, 1)
    self.gridLayout_13.addWidget(self.groupBox_5, 0, 0, 1, 1)
    self.widget_6 = QtWidgets.QWidget(self.tab_4)
    sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
    sizePolicy.setHorizontalStretch(0)
    sizePolicy.setVerticalStretch(1)
    sizePolicy.setHeightForWidth(self.widget_6.sizePolicy().hasHeightForWidth())
    self.widget_6.setSizePolicy(sizePolicy)
    self.widget_6.setObjectName("widget_6")
    self.gridLayout_12 = QtWidgets.QGridLayout(self.widget_6)
    self.gridLayout_12.setContentsMargins(3, 3, 3, 3)
    self.gridLayout_12.setObjectName("gridLayout_12")
    self.OpaqueAllCheckbox = QtWidgets.QCheckBox(self.widget_6)
    self.OpaqueAllCheckbox.setObjectName("OpaqueAllCheckbox")
    self.gridLayout_12.addWidget(self.OpaqueAllCheckbox, 0, 0, 1, 1)
    self.binstable = QtWidgets.QTableWidget(self.widget_6)
    sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
    sizePolicy.setHorizontalStretch(0)
    sizePolicy.setVerticalStretch(1)
    sizePolicy.setHeightForWidth(self.binstable.sizePolicy().hasHeightForWidth())
    self.binstable.setSizePolicy(sizePolicy)
    self.binstable.setRowCount(5)
    self.binstable.setColumnCount(4)
    self.binstable.setObjectName("binstable")
    self.binstable.setColumnCount(4)
    self.binstable.setRowCount(5)
    self.gridLayout_12.addWidget(self.binstable, 1, 0, 1, 1)
    self.gridLayout_13.addWidget(self.widget_6, 1, 0, 1, 1)
    self.functionTabWidget.addTab(self.tab_4, "")
    self.tab_5 = QtWidgets.QWidget()
    self.tab_5.setObjectName("tab_5")
    self.gridLayout_8 = QtWidgets.QGridLayout(self.tab_5)
    self.gridLayout_8.setObjectName("gridLayout_8")
    self.groupBox_6 = QtWidgets.QGroupBox(self.tab_5)
    sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
    sizePolicy.setHorizontalStretch(0)
    sizePolicy.setVerticalStretch(0)
    sizePolicy.setHeightForWidth(self.groupBox_6.sizePolicy().hasHeightForWidth())
    self.groupBox_6.setSizePolicy(sizePolicy)
    self.groupBox_6.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
    self.groupBox_6.setObjectName("groupBox_6")
    self.gridLayout_21 = QtWidgets.QGridLayout(self.groupBox_6)
    self.gridLayout_21.setObjectName("gridLayout_21")
    self.DrawReciprocUnitCellBox = QtWidgets.QGroupBox(self.groupBox_6)
    self.DrawReciprocUnitCellBox.setCheckable(True)
    self.DrawReciprocUnitCellBox.setChecked(False)
    self.DrawReciprocUnitCellBox.setObjectName("DrawReciprocUnitCellBox")
    self.gridLayout_25 = QtWidgets.QGridLayout(self.DrawReciprocUnitCellBox)
    self.gridLayout_25.setObjectName("gridLayout_25")
    self.label_2 = QtWidgets.QLabel(self.DrawReciprocUnitCellBox)
    sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
    sizePolicy.setHorizontalStretch(0)
    sizePolicy.setVerticalStretch(0)
    sizePolicy.setHeightForWidth(self.label_2.sizePolicy().hasHeightForWidth())
    self.label_2.setSizePolicy(sizePolicy)
    self.label_2.setWordWrap(True)
    self.label_2.setObjectName("label_2")
    self.gridLayout_25.addWidget(self.label_2, 0, 0, 1, 1)
    self.reciprocunitcellslider = QtWidgets.QSlider(self.DrawReciprocUnitCellBox)
    sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
    sizePolicy.setHorizontalStretch(0)
    sizePolicy.setVerticalStretch(0)
    sizePolicy.setHeightForWidth(self.reciprocunitcellslider.sizePolicy().hasHeightForWidth())
    self.reciprocunitcellslider.setSizePolicy(sizePolicy)
    self.reciprocunitcellslider.setMinimum(0)
    self.reciprocunitcellslider.setMaximum(100)
    self.reciprocunitcellslider.setOrientation(QtCore.Qt.Horizontal)
    self.reciprocunitcellslider.setObjectName("reciprocunitcellslider")
    self.gridLayout_25.addWidget(self.reciprocunitcellslider, 0, 1, 1, 1)
    self.label_4 = QtWidgets.QLabel(self.DrawReciprocUnitCellBox)
    self.label_4.setWordWrap(True)
    self.label_4.setObjectName("label_4")
    self.gridLayout_25.addWidget(self.label_4, 0, 2, 1, 1)
    self.gridLayout_21.addWidget(self.DrawReciprocUnitCellBox, 1, 0, 1, 1)
    self.DrawRealUnitCellBox = QtWidgets.QGroupBox(self.groupBox_6)
    self.DrawRealUnitCellBox.setCheckable(True)
    self.DrawRealUnitCellBox.setChecked(False)
    self.DrawRealUnitCellBox.setObjectName("DrawRealUnitCellBox")
    self.gridLayout_26 = QtWidgets.QGridLayout(self.DrawRealUnitCellBox)
    self.gridLayout_26.setObjectName("gridLayout_26")
    self.label_6 = QtWidgets.QLabel(self.DrawRealUnitCellBox)
    self.label_6.setWordWrap(True)
    self.label_6.setObjectName("label_6")
    self.gridLayout_26.addWidget(self.label_6, 0, 0, 1, 1)
    self.unitcellslider = QtWidgets.QSlider(self.DrawRealUnitCellBox)
    self.unitcellslider.setMinimum(0)
    self.unitcellslider.setMaximum(100)
    self.unitcellslider.setOrientation(QtCore.Qt.Horizontal)
    self.unitcellslider.setObjectName("unitcellslider")
    self.gridLayout_26.addWidget(self.unitcellslider, 0, 1, 1, 1)
    self.label_5 = QtWidgets.QLabel(self.DrawRealUnitCellBox)
    self.label_5.setWordWrap(True)
    self.label_5.setObjectName("label_5")
    self.gridLayout_26.addWidget(self.label_5, 0, 2, 1, 1)
    self.gridLayout_21.addWidget(self.DrawRealUnitCellBox, 2, 0, 1, 1)
    self.frame_4 = QtWidgets.QFrame(self.groupBox_6)
    sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
    sizePolicy.setHorizontalStretch(0)
    sizePolicy.setVerticalStretch(0)
    sizePolicy.setHeightForWidth(self.frame_4.sizePolicy().hasHeightForWidth())
    self.frame_4.setSizePolicy(sizePolicy)
    self.frame_4.setFrameShape(QtWidgets.QFrame.StyledPanel)
    self.frame_4.setFrameShadow(QtWidgets.QFrame.Raised)
    self.frame_4.setObjectName("frame_4")
    self.gridLayout_14 = QtWidgets.QGridLayout(self.frame_4)
    self.gridLayout_14.setObjectName("gridLayout_14")
    self.ResetViewBtn = QtWidgets.QPushButton(self.frame_4)
    sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
    sizePolicy.setHorizontalStretch(0)
    sizePolicy.setVerticalStretch(0)
    sizePolicy.setHeightForWidth(self.ResetViewBtn.sizePolicy().hasHeightForWidth())
    self.ResetViewBtn.setSizePolicy(sizePolicy)
    self.ResetViewBtn.setObjectName("ResetViewBtn")
    self.gridLayout_14.addWidget(self.ResetViewBtn, 0, 0, 1, 1)
    self.SaveImageBtn = QtWidgets.QPushButton(self.frame_4)
    sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
    sizePolicy.setHorizontalStretch(0)
    sizePolicy.setVerticalStretch(0)
    sizePolicy.setHeightForWidth(self.SaveImageBtn.sizePolicy().hasHeightForWidth())
    self.SaveImageBtn.setSizePolicy(sizePolicy)
    self.SaveImageBtn.setObjectName("SaveImageBtn")
    self.gridLayout_14.addWidget(self.SaveImageBtn, 0, 1, 1, 1)
    self.gridLayout_21.addWidget(self.frame_4, 0, 0, 1, 1)
    self.gridLayout_8.addWidget(self.groupBox_6, 0, 0, 1, 1)
    self.functionTabWidget.addTab(self.tab_5, "")
    self.textInfo = QtWidgets.QPlainTextEdit(self.splitter_2)
    sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
    sizePolicy.setHorizontalStretch(0)
    sizePolicy.setVerticalStretch(2)
    sizePolicy.setHeightForWidth(self.textInfo.sizePolicy().hasHeightForWidth())
    self.textInfo.setSizePolicy(sizePolicy)
    self.textInfo.setVerticalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOn)
    self.textInfo.setHorizontalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOn)
    self.textInfo.setLineWrapMode(QtWidgets.QPlainTextEdit.NoWrap)
    self.textInfo.setObjectName("textInfo")
    self.gridLayout_4.addWidget(self.splitter_2, 0, 0, 1, 1)
    self.BrowserBox = QWebEngineView(self.splitter)
    sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
    sizePolicy.setHorizontalStretch(1)
    sizePolicy.setVerticalStretch(0)
    sizePolicy.setHeightForWidth(self.BrowserBox.sizePolicy().hasHeightForWidth())
    self.BrowserBox.setSizePolicy(sizePolicy)
    self.BrowserBox.setObjectName("BrowserBox")
    self.gridLayout.addWidget(self.splitter, 0, 0, 1, 1)
    self.gridLayout_2.addWidget(self.widget, 0, 0, 1, 1)
    MainWindow.setCentralWidget(self.centralwidget)
    self.menubar = QtWidgets.QMenuBar(MainWindow)
    self.menubar.setGeometry(QtCore.QRect(0, 0, 1066, 18))
    self.menubar.setObjectName("menubar")
    self.menuFile = QtWidgets.QMenu(self.menubar)
    self.menuFile.setObjectName("menuFile")
    MainWindow.setMenuBar(self.menubar)
    self.statusBar = QtWidgets.QStatusBar(MainWindow)
    self.statusBar.setObjectName("statusBar")
    MainWindow.setStatusBar(self.statusBar)
    self.actionOpen_reflection_file = QtWidgets.QAction(MainWindow)
    self.actionOpen_reflection_file.setObjectName("actionOpen_reflection_file")
    self.actionSettings = QtWidgets.QAction(MainWindow)
    self.actionSettings.setObjectName("actionSettings")
    self.actiondebug = QtWidgets.QAction(MainWindow)
    self.actiondebug.setObjectName("actiondebug")
    self.actionExit = QtWidgets.QAction(MainWindow)
    self.actionExit.setObjectName("actionExit")
    self.actionSave_reflection_file = QtWidgets.QAction(MainWindow)
    self.actionSave_reflection_file.setObjectName("actionSave_reflection_file")
    self.menuFile.addAction(self.actionOpen_reflection_file)
    self.menuFile.addAction(self.actionSave_reflection_file)
    self.menuFile.addAction(self.actionSettings)
    self.menuFile.addAction(self.actiondebug)
    self.menuFile.addAction(self.actionExit)
    self.menubar.addAction(self.menuFile.menuAction())

    self.retranslateUi(MainWindow)
    self.functionTabWidget.setCurrentIndex(0)
    QtCore.QMetaObject.connectSlotsByName(MainWindow)

  def retranslateUi(self, MainWindow):
    MainWindow.setWindowTitle(QtWidgets.QApplication.translate("MainWindow", "MainWindow", None, -1))
    self.label.setText(QtWidgets.QApplication.translate("MainWindow", "Display a data set with a double-click. Right-click table for more options.", None, -1))
    self.ExpandReflsGroupBox.setTitle(QtWidgets.QApplication.translate("MainWindow", "Expand reflections", None, -1))
    self.expandP1checkbox.setText(QtWidgets.QApplication.translate("MainWindow", "Expand to P1", None, -1))
    self.expandAnomalouscheckbox.setText(QtWidgets.QApplication.translate("MainWindow", "Show Friedel pairs", None, -1))
    self.SpacegroupLabel.setText(QtWidgets.QApplication.translate("MainWindow", "Space Subgroups", None, -1))
    self.sysabsentcheckbox.setText(QtWidgets.QApplication.translate("MainWindow", "Show Systematic Absences", None, -1))
    self.missingcheckbox.setText(QtWidgets.QApplication.translate("MainWindow", "Show Missing Reflections", None, -1))
    self.onlymissingcheckbox.setText(QtWidgets.QApplication.translate("MainWindow", "Only", None, -1))
    self.functionTabWidget.setTabText(self.functionTabWidget.indexOf(self.tab), QtWidgets.QApplication.translate("MainWindow", "Expansion", None, -1))
    self.SliceReflectionsBox.setText(QtWidgets.QApplication.translate("MainWindow", "Slice Reflections", None, -1))
    self.fixedorientcheckbox.setText(QtWidgets.QApplication.translate("MainWindow", "Fix orientation but allow zoom and translation", None, -1))
    self.ClipPlaneChkGroupBox.setTitle(QtWidgets.QApplication.translate("MainWindow", "with a clip plane oriented", None, -1))
    self.clipNormalBtn.setText(QtWidgets.QApplication.translate("MainWindow", "Perpendicular to vector below", None, -1))
    self.clipParallelBtn.setText(QtWidgets.QApplication.translate("MainWindow", "Parallel to vector below", None, -1))
    self.groupBox_2.setTitle(QtWidgets.QApplication.translate("MainWindow", "Vector Components (r1, r2, r3) are specified", None, -1))
    self.recipvecBtn.setText(QtWidgets.QApplication.translate("MainWindow", "in reciprocal unit cell coordinates", None, -1))
    self.clipTNCSBtn.setText(QtWidgets.QApplication.translate("MainWindow", "as tNCS vector", None, -1))
    self.realspacevecBtn.setText(QtWidgets.QApplication.translate("MainWindow", "in real space unit cell coordinates", None, -1))
    self.rotavecangle_labeltxt.setText(QtWidgets.QApplication.translate("MainWindow", "Angle rotated around vector ", None, -1))
    self.label_21.setText(QtWidgets.QApplication.translate("MainWindow", "r1:", None, -1))
    self.label_22.setText(QtWidgets.QApplication.translate("MainWindow", "r2:", None, -1))
    self.label_23.setText(QtWidgets.QApplication.translate("MainWindow", "r3:", None, -1))
    self.label_15.setText(QtWidgets.QApplication.translate("MainWindow", "Clip Plane Width:", None, -1))
    self.label_14.setText(QtWidgets.QApplication.translate("MainWindow", "Distance from origin:", None, -1))
    self.showsliceGroupCheckbox.setTitle(QtWidgets.QApplication.translate("MainWindow", "Explicitly with a plane at", None, -1))
    self.label_3.setText(QtWidgets.QApplication.translate("MainWindow", "equal to", None, -1))
    self.functionTabWidget.setTabText(self.functionTabWidget.indexOf(self.tab_2), QtWidgets.QApplication.translate("MainWindow", "Slicing", None, -1))
    self.groupBox_3.setTitle(QtWidgets.QApplication.translate("MainWindow", "Radii size of HKL spheres", None, -1))
    self.label_7.setText(QtWidgets.QApplication.translate("MainWindow", "<html><head/><body><p>Tick the &quot;User defined power scaling&quot; box for chosing a power factor. Untick this box for an automatically computed power factor. Automatic power factor will render reflections so that the reflection with the smallest data value is rendered about 10 times smaller than the reflection with the largest data value. <br/>Entering a power factor of 1 means the size of a sphere for each reflection is proportional to its data value. For a typical data set of intensities with values differing by some orders of magnitude only the strongest reflections would then be visible. Entering a power factor of 0 means all spheres of reflections will have the same size irrespective of their data values. This means even very weak reflections would then be visible. </p></body></html>", None, -1))
    self.ManualPowerScalecheckbox.setText(QtWidgets.QApplication.translate("MainWindow", "User defined power scaling", None, -1))
    self.label_10.setText(QtWidgets.QApplication.translate("MainWindow", "Power scale factor:", None, -1))
    self.label_11.setToolTip(QtWidgets.QApplication.translate("MainWindow", "<html><head/><body><p>A linear scale factor of 1 means that if two reflectiions with the largest data values happen to be next to each other their spheres will touch but not overlap.</p></body></html>", None, -1))
    self.label_11.setText(QtWidgets.QApplication.translate("MainWindow", "Linear scale factor", None, -1))
    self.radii_scale_spinBox.setToolTip(QtWidgets.QApplication.translate("MainWindow", "<html><head/><body><p>A linear scale factor of 1 means that if two reflectiions with the largest data values happen to be next to each other their spheres will touch but not overlap.</p></body></html>", None, -1))
    self.functionTabWidget.setTabText(self.functionTabWidget.indexOf(self.tab_3), QtWidgets.QApplication.translate("MainWindow", "Sizing", None, -1))
    self.groupBox_5.setTitle(QtWidgets.QApplication.translate("MainWindow", "Bin according to", None, -1))
    self.BinDataComboBox.setToolTip(QtWidgets.QApplication.translate("MainWindow", "<html><head/><body><p>Order the curently displayed data set in bins according to the values of the selected data set. Reflections in the currently displayed data set not matching any reflections in the selected data set will be put in a bin labelled with &quot;NaN&quot;.</p></body></html>", None, -1))
    self.label_12.setText(QtWidgets.QApplication.translate("MainWindow", "Number of bins:", None, -1))
    self.Nbins_spinBox.setToolTip(QtWidgets.QApplication.translate("MainWindow", "<html><head/><body><p>Select the number of bins for ordering the currently displayed data set. More than about 10-15 bins will slow down the HKLviewer when working with large data sets.</p></body></html>", None, -1))
    self.OpaqueAllCheckbox.setText(QtWidgets.QApplication.translate("MainWindow", "Show/hide all data", None, -1))
    self.binstable.setToolTip(QtWidgets.QApplication.translate("MainWindow", "<html><head/><body><p>Change the visibility of selected bins either by ticking or unticking the check boxes or by entering an opacity value between 0 and 1</p></body></html>", None, -1))
    self.functionTabWidget.setTabText(self.functionTabWidget.indexOf(self.tab_4), QtWidgets.QApplication.translate("MainWindow", "Binning", None, -1))
    self.groupBox_6.setTitle(QtWidgets.QApplication.translate("MainWindow", "Geometry", None, -1))
    self.DrawReciprocUnitCellBox.setToolTip(QtWidgets.QApplication.translate("MainWindow", "<html><head/><body><p>The reciprocal unit cell spanning a*,b*,c* from (0,0,0) to (1,1,1) is located in the centre of the displayed reflections. and is therefore quite small. Adjust the slider to scale its outline to become compatible with the sphere of displayed reflections.</p></body></html>", None, -1))
    self.DrawReciprocUnitCellBox.setTitle(QtWidgets.QApplication.translate("MainWindow", "Show Reciprocal Unit Cell", None, -1))
    self.label_2.setText(QtWidgets.QApplication.translate("MainWindow", "True to Scale of HKL grid", None, -1))
    self.label_4.setText(QtWidgets.QApplication.translate("MainWindow", "Close to radius of sphere of reflections", None, -1))
    self.DrawRealUnitCellBox.setToolTip(QtWidgets.QApplication.translate("MainWindow", "<html><head/><body><p>The real space unit cell is often much larger than the sphere of displayed reflections because the reciprocal unit cell size is inversely proportional to the size of the real space unit cell. Adjust the slider to scale its outline to become compatible with the sphere of displayed reflections.</p></body></html>", None, -1))
    self.DrawRealUnitCellBox.setTitle(QtWidgets.QApplication.translate("MainWindow", "Show Real Space Unit Cell", None, -1))
    self.label_6.setText(QtWidgets.QApplication.translate("MainWindow", "True to Scale of HKL grid", None, -1))
    self.label_5.setText(QtWidgets.QApplication.translate("MainWindow", "Close to radius of sphere of reflections", None, -1))
    self.ResetViewBtn.setToolTip(QtWidgets.QApplication.translate("MainWindow", "<html><head/><body><p>Place the displayed reflections in the centre of the view.</p></body></html>", None, -1))
    self.ResetViewBtn.setText(QtWidgets.QApplication.translate("MainWindow", "Reset View", None, -1))
    self.SaveImageBtn.setText(QtWidgets.QApplication.translate("MainWindow", "Save Image", None, -1))
    self.functionTabWidget.setTabText(self.functionTabWidget.indexOf(self.tab_5), QtWidgets.QApplication.translate("MainWindow", "Extras", None, -1))
    self.menuFile.setTitle(QtWidgets.QApplication.translate("MainWindow", "File", None, -1))
    self.actionOpen_reflection_file.setText(QtWidgets.QApplication.translate("MainWindow", "Open reflection file...", None, -1))
    self.actionSettings.setText(QtWidgets.QApplication.translate("MainWindow", "Settings", None, -1))
    self.actiondebug.setText(QtWidgets.QApplication.translate("MainWindow", "Debug", None, -1))
    self.actionExit.setText(QtWidgets.QApplication.translate("MainWindow", "Exit", None, -1))
    self.actionSave_reflection_file.setText(QtWidgets.QApplication.translate("MainWindow", "Save reflection file", None, -1))




from PySide2.QtWebEngineWidgets import QWebEngineView
try: # if invoked by cctbx.python or some such
  from crys3d.hklview.helpers import HeaderDataTableWidget
except Exception as e: # if invoked by a generic python that doesn't know cctbx modules
  from helpers import HeaderDataTableWidget




"""
Module for querying the RCSB web server using the REST API, as described here:
http://www.rcsb.org/pdb/software/rest.do

There is some overlap with iotbx.pdb.fetch, which really should have gone here
instead, but this module is intended to be used in higher-level automation
pipelines.
"""

from __future__ import absolute_import, division, print_function
import libtbx.utils
from xml.dom.minidom import parseString
import sys

url_base = "http://www.rcsb.org/pdb/rest"
url_search = url_base + "/search"

def post_query(query_xml, xray_only=True, d_max=None, d_min=None,
    protein_only=False, data_only=False, identity_cutoff=None, log=None,
    sort_by_resolution=False):
  """
  Generate the full XML for a multi-part query with generic search options,
  starting from the basic query passed by another function, post it to the
  RCSB's web service, and return a list of matching PDB IDs.

  Parameters
  ----------
  query_xml : str or None
  xray_only : bool, optional
  d_max : float, optional
  d_min : float, optional
  protein_only : bool, optional
  data_only : bool, optional
  identity_cutoff : int, optional
      Apply sequence filtering to remove structures with greater than this
      percentage sequence identity. Must be one of 100, 95, 90, 70, 50, 40, 30.
  log : file, optional
  sort_by_resolution : bool, optional
  """
  if (log is None):
    log = libtbx.utils.null_out()
  queries = []
  if (query_xml is not None):
    queries.append(query_xml)
  print("Setting up RCSB server query:", file=log)
  if (xray_only):
    print("  limiting to X-ray structures", file=log)
    xray_query = "<queryType>org.pdb.query.simple.ExpTypeQuery</queryType>\n"+\
      "<mvStructure.expMethod.value>X-RAY</mvStructure.expMethod.value>"
    if (data_only):
      xray_query += "\n<mvStructure.hasExperimentalData.value>Y</mvStructure.hasExperimentalData.value>"
    queries.append(xray_query)
  if (d_max is not None) or (d_min is not None):
    base_clause = "<queryType>org.pdb.query.simple.ResolutionQuery</queryType>"
    base_clause += "\n<refine.ls_d_res_high.comparator>between" + \
      "</refine.ls_d_res_high.comparator>"
    if (d_min is not None):
      assert (d_min >= 0)
      print("  applying resolution cutoff d_min > %f" % d_min, file=log)
      base_clause += \
        "\n<refine.ls_d_res_high.min>%f</refine.ls_d_res_high.min>" % d_min
    if (d_max is not None):
      assert (d_max >= 0)
      print("  applying resolution cutoff d_min < %f" % d_max, file=log)
      base_clause += \
        "\n<refine.ls_d_res_high.max>%f</refine.ls_d_res_high.max>" % d_max
    queries.append(base_clause)
  if (protein_only):
    print("  excluding non-protein models", file=log)
    queries.append(
      "<queryType>org.pdb.query.simple.ChainTypeQuery</queryType>\n" +
      "<containsProtein>Y</containsProtein>")
  if (identity_cutoff is not None):
    print("  filtering based on sequence identity < %d %%" % \
      int(identity_cutoff), file=log)
    assert (identity_cutoff > 0) and (identity_cutoff <= 100)
    queries.append(
      "<queryType>org.pdb.query.simple.HomologueReductionQuery</queryType>\n"+
      "<identityCutoff>%d</identityCutoff>" % int(identity_cutoff))
  if (len(queries) == 0):
    raise RuntimeError("No queries specified!")
  queries_string = ""
  k = 0
  for clause in queries :
    queries_string += """
        <queryRefinement>
          <queryRefinementLevel>%d</queryRefinementLevel>
          <conjunctionType>and</conjunctionType>
          <orgPdbQuery>
            %s
          </orgPdbQuery>
        </queryRefinement>""" % (k, clause)
    k += 1
  query_str = """\
<orgPdbCompositeQuery version="1.0">
  %s
</orgPdbCompositeQuery>
""" % (queries_string)
  parsed = parseString(query_str)
  url_final = url_search
  if (sort_by_resolution):
    url_final += "/?sortfield=Resolution"
    print("  will sort by resolution", file=log)
  print("  executing HTTP request...", file=log)
  result = libtbx.utils.urlopen(url_final, query_str).read()
  return result.splitlines()

def sequence_search(sequence, **kwds):
  search_type = kwds.pop("search_type", "blast")
  expect = kwds.pop("expect", 0.01)
  min_identity = kwds.pop("min_identity", 0.0)
  if (min_identity <= 1):
    min_identity *= 100
  """
  Homology search for an amino acid sequence.  The advantage of using this
  service over the NCBI/EBI BLAST servers (in iotbx.pdb.fetch) is the ability
  to exclude non-Xray structures.
  """
  assert (search_type in ["blast", "fasta", "psiblast"])
  query_str = """\
    <queryType>org.pdb.query.simple.SequenceQuery</queryType>
      <description>Sequence Search (expect = %g, search = %s)</description>
      <sequence>%s</sequence>
      <eCutOff>%g</eCutOff>
      <sequenceIdentityCutoff>%g</sequenceIdentityCutoff>
    <searchTool>%s</searchTool>""" % (expect, search_type, sequence, expect,
      min_identity, search_type)
  return post_query(query_str, **kwds)

def chemical_id_search(resname, **kwds):
  """
  Find all entry IDs with the specified chemical ID.

  Parameters
  ----------
  resname : str
  kwds : dict
      Keyword arguments passed to post_query.

  Examples
  --------
  >>> from mmtbx.wwpdb.rcsb_web_services import chemical_id_search
  >>> len(chemical_id_search("ZN", data_only=True, identity_cutoff=70))
  2874
  """
  assert (1 <= len(resname) <= 3)
  polymeric_type = kwds.pop("polymeric_type", "Any")
  assert (polymeric_type in ["Any", "Free", "Polymeric"])
  polymer_limit = "<polymericType>%s</polymericType>" % polymeric_type
  query_str = """\
    <queryType>org.pdb.query.simple.ChemCompIdQuery</queryType>
    <description>Chemical ID: %s</description>
    <chemCompId>%s</chemCompId>
    %s""" % (resname, resname, polymer_limit)
  return post_query(query_str, **kwds)

def get_custom_report_table(pdb_ids, columns, log=sys.stdout,
    prefix="dimStructure", check_for_missing=True):
  """Given a list of PDB IDs and a list of attribute identifiers, returns a
  Python list of lists for the IDs and attributes."""
  assert (len(columns) > 0)
  if (len(pdb_ids) == 0) : return []
  url_base = "http://www.rcsb.org/pdb/rest/customReport?"
  url = url_base + "pdbids=%s" % ",".join(pdb_ids)
  all_columns = ["structureId"] + columns
  url += "&customReportColumns=%s" % ",".join(all_columns)
  result = libtbx.utils.urlopen(url).read()
  # The RCSB's custom report follows this format (using the high-resolution
  # limit as an example):
  #
  # <?xml version='1.0' standalone='no' ?>
  # <dataset>
  #   <record>
  #     <dimStructure.structureId>1A0I</dimStructure.structureId>
  #     <dimStructure.highResolutionLimit>2.6</dimStructure.highResolutionLimit>
  #   </record>
  # </dataset>
  #
  # XXX note that 'dimStructure' is substituted for by 'dimEntity' in the
  # ligand report (and probably others), and that this table will not
  # necessarily have one row per PDB ID.
  xmlrec = parseString(result)
  table = []
  report_ids = set([])
  records = xmlrec.getElementsByTagName("record")
  for record in records :
    pdb_id_nodes = record.getElementsByTagName("%s.structureId" % prefix)
    assert (len(pdb_id_nodes) == 1), record.toxml()
    pdb_id = pdb_id_nodes[0].childNodes[0].data
    report_ids.add(pdb_id)
    row = [ pdb_id ] # pdb_id
    for col_name in columns :
      row_col = record.getElementsByTagName("%s.%s" % (prefix, col_name))
      assert (len(row_col) == 1)
      row.append(row_col[0].childNodes[0].data)
    table.append(row)
  if (check_for_missing):
    missing_ids = set(pdb_ids) - report_ids
    if (len(missing_ids) > 0):
      print("WARNING: missing report info for %d IDs:"%len(missing_ids), file=log)
      print("  %s" % " ".join(sorted(list(missing_ids))), file=log)
  return table

def get_high_resolution_for_structures(pdb_ids):
  return get_custom_report_table(pdb_ids, columns=["highResolutionLimit"])

def get_high_resolution_and_residue_count_for_structures(pdb_ids):
  return get_custom_report_table(pdb_ids, columns=["highResolutionLimit", 'residueCount'])

def get_ligand_info_for_structures(pdb_ids):
  """Return a list of ligands in the specified structure(s), including the
  SMILES strings."""
  return get_custom_report_table(pdb_ids,
    columns=["chainId","ligandId","ligandMolecularWeight","ligandFormula",
      "ligandName","ligandSmiles"],
    prefix="dimEntity",
    check_for_missing=False)



from __future__ import division, print_function
from cctbx.array_family import flex
import iotbx.pdb
import mmtbx.model
import iotbx.mrcfile
import boost_adaptbx.boost.python as bp
cctbx_maptbx_ext = bp.import_ext("cctbx_maptbx_ext")
import time
from cctbx import maptbx
from libtbx import adopt_init_args
from libtbx.utils import user_plus_sys_time
from cctbx import crystal, maptbx, xray, adptbx
from iotbx.pdb.utils import all_chain_ids
import scitbx.math
from libtbx.utils import Sorry

elements=["N","O","S","Se","Fe","Mn","Mg","Mn","Zn","Ca","Cd","Cu","Co"]
selection_string_interaction = " or ".join(["element %s"%e for e in elements])

hoh_str = "ATOM  %5d  O   HOH S%4d    %8.3f%8.3f%8.3f  1.00  0.00           O"

def _debug_show_all_plots(
      map_data,
      unit_cell,
      center_cart,
      radius,
      plot_number,
      s_angle_sampling_step = 60,
      t_angle_sampling_step = 60):
  rho = flex.double()
  vecs = []
  one_d = flex.double()
  for s in range(0,360,s_angle_sampling_step):
    for t in range(0,360,t_angle_sampling_step):
      xc,yc,zc = scitbx.math.point_on_sphere(r=radius, s_deg=s, t_deg=t,
        center=center_cart)
      xf,yf,zf = unit_cell.fractionalize([xc,yc,zc])
      rho.append(map_data.tricubic_interpolation([xf,yf,zf]))
      ###
      v = flex.double()
      r = flex.double()
      for p in [p/20 for p in range(0,21)]:
        xp = center_cart[0] + p * (xc-center_cart[0])
        yp = center_cart[1] + p * (yc-center_cart[1])
        zp = center_cart[2] + p * (zc-center_cart[2])
        xpf,ypf,zpf = unit_cell.fractionalize([xp,yp,zp])
        v.append(map_data.tricubic_interpolation([xpf,ypf,zpf]))
        r.append(p)
      vecs.append([r,v])
      one_d.extend(v)
      ###
  import matplotlib.pyplot as plt
  import matplotlib as mpl
  mpl.use('Agg')
  fig, axes = plt.subplots(ncols=6, nrows=6)
  for i, ax in enumerate(axes.flatten()):
    ax.plot(vecs[i][0], vecs[i][1])
    ax.set_ylim(flex.min(one_d),flex.max(one_d))
    ax.set_xticks([])
    ax.set_yticks([])
  fig.savefig("fig_%s.png"%plot_number)
  ###
  del plt
  del mpl
  return None

class msg_accumulator(object):
  def __init__(self, log=None):
    self.log = log
    self.messages = []

  def add(self, msg):
    self.messages.append(msg)
    if(self.log is not None): print(msg, file=self.log)

  def show(self, log=None):
    assert [self.log, log].count(None) == 1
    if(log is not None):
      assert self.log is None
    else:
      assert self.log is not None
      log = self.log
    for msg in self.messages:
      print(msg, file=log)

class run(object):
  def __init__(self,
               model,
               map_data,
               dist_min=2.0,
               dist_max=3.2,
               step=0.3,
               mean_scale=0.7,
               debug=False,
               log=None):
    # Common parameters / variables
    adopt_init_args(self, locals())
    self.total_time = 0
    if(map_data.accessor().origin()!=(0,0,0)):
      raise Sorry("Map origin must be zero.")
    self.ma = msg_accumulator(log = log)
    self.map_data = self.map_data.deep_copy() # will be changed in-place!
    self.unit_cell = self.model.crystal_symmetry().unit_cell()
    self.sites_frac = self.model.get_xray_structure().sites_frac()
    self.sites_frac_interaction = self.sites_frac.select(
      self.model.selection(selection_string_interaction))
    self.sites_frac_water = None
    self.cutoff = None
    self._wrote_file=0
    # Calculations
    self._call(self._massage_map         , "Normalize and re-sample input map")
    self._call(self._get_cutoff          , "Get cutoff")
    self._call(self._mask_out            , "Mask out molecule/solvent regions")
    self._call(self._find_peaks          , "Find peaks")
    self._call(self._filter_by_distance  , "Filter peaks by distance")
    self._call(self._filter_by_sphericity, "Filter peaks by sphericity")
    self._call(self._filter_by_distance  , "Filter peaks by distance")
    self._call(self._append_to_model     , "Add to model and reset internals")
    if(self.debug):
      self._call(self._show_peak_profiles, "Show peak profiles")

  def _call(self, func, msg):
    timer = user_plus_sys_time()
    self.ma.add(msg)
    func()
    t = timer.elapsed()
    self.total_time += t
    self.ma.add("  time (s): %s (total time: %s)"%(("%8.3f"%t).strip(),
      ("%8.3f"%self.total_time).strip()))

  def _map_mmm_str(self):
    return " ".join([("%8.3f"%m).strip() for m in [
      flex.min(self.map_data),
      flex.max(self.map_data),
      flex.mean(self.map_data)]])

  def _write_map(self, file_name):
    iotbx.mrcfile.write_ccp4_map(
      file_name   = file_name,
      unit_cell   = self.unit_cell,
      space_group = self.model.crystal_symmetry().space_group(),
      map_data    = self.map_data.as_double(),
      labels      = flex.std_string([""]))

  def _massage_map(self):
    self.ma.add("  input map (min,max,mean): %s"%self._map_mmm_str())
    #
    self.map_data = self.map_data - flex.mean(self.map_data)
    self.map_data = self.map_data.set_selected(self.map_data < 0, 0)
    sd = self.map_data.sample_standard_deviation()
    assert sd != 0
    self.map_data = self.map_data / sd
    self.ma.add("  re-scaled map (min,max,mean): %s"%self._map_mmm_str())
    if(self.debug):
      self._write_map(file_name = "rescaled.mrc")
    #
    a,b,c = self.unit_cell.parameters()[:3]
    n_real = self.map_data.accessor().all()
    sx, sy, sz = a/n_real[0], b/n_real[1], c/n_real[2]
    self.ma.add("  input map dimenstions: %d %d %d"%n_real)
    self.ma.add("  input map grid steps (A): %s %s %s"%(
      ("%6.3f"%sx).strip(), ("%6.3f"%sy).strip(), ("%6.3f"%sz).strip()))
    if(max(sx, sy, sz) > self.step):
      n_real_fine = (int(a/self.step), int(b/self.step), int(c/self.step))
      self.ma.add("  re-sampled map dimenstions: %d %d %d"%n_real_fine)
      map_fine = flex.double(flex.grid(n_real_fine), 0)
      maptbx.resample(
        map_data     = self.map_data,
        map_data_new = map_fine,
        unit_cell    = self.unit_cell)
      self.map_data = map_fine
      self.ma.add("  input map (min,max,mean): %s"%self._map_mmm_str())
    if(self.debug):
      self._write_map(file_name = "resampled.mrc")

  def _write_pdb_file(self, file_name_prefix):
    file_name = file_name_prefix + "_%d.pdb"%self._wrote_file
    assert self.sites_frac_water is not None
    with open(file_name, "w") as fo:
      for i, site_frac in enumerate(self.sites_frac_water):
        site_cart = self.unit_cell.orthogonalize(site_frac)
        print(hoh_str%(i,i,site_cart[0],site_cart[1],site_cart[2]), file=fo)
    self._wrote_file += 1

  def _get_cutoff(self):
    map_at_atoms = flex.double()
    for site_frac in self.sites_frac:
      map_at_atoms.append( self.map_data.tricubic_interpolation(site_frac) )
    mean = flex.mean(map_at_atoms)
    sel = map_at_atoms > mean/3 # so it is not spoiled by too low/high values
    self.cutoff = flex.mean( map_at_atoms.select(sel) )*self.mean_scale
    self.ma.add("  mean density at atom centers: %8.3f"%mean)
    self.ma.add("  density cutoff for peak search: %8.3f"%self.cutoff)

  def _mask_out(self, rad_inside = 1.0, rad_outside = 6.0):
    # Zero inside
    radii = flex.double(self.sites_frac.size(), rad_inside)
    mask = cctbx_maptbx_ext.mask(
      sites_frac                  = self.sites_frac,
      unit_cell                   = self.unit_cell,
      n_real                      = self.map_data.all(),
      mask_value_inside_molecule  = 0,
      mask_value_outside_molecule = 1,
      radii                       = radii)
    self.map_data = self.map_data * mask
    # Zero outside
    radii = flex.double(self.sites_frac_interaction.size(), rad_outside)
    mask = cctbx_maptbx_ext.mask(
      sites_frac                  = self.sites_frac_interaction,
      unit_cell                   = self.unit_cell,
      n_real                      = self.map_data.all(),
      mask_value_inside_molecule  = 1,
      mask_value_outside_molecule = 0,
      radii                       = radii)
    self.map_data = self.map_data * mask

  def _find_peaks(self, min_peak_peak_distance = 1.5):
    cg = maptbx.crystal_gridding(
      space_group_info      = self.model.crystal_symmetry().space_group_info(),
      symmetry_flags        = maptbx.use_space_group_symmetry,
      unit_cell             = self.unit_cell,
      pre_determined_n_real = self.map_data.accessor().all())
    cgt = maptbx.crystal_gridding_tags(gridding = cg)
    peak_search_parameters = maptbx.peak_search_parameters(
      peak_search_level      = 3,
      max_peaks              = 0,
      peak_cutoff            = self.cutoff,
      interpolate            = True,
      min_distance_sym_equiv = 0,
      general_positions_only = False, # ???????XXXXXXXXXXXXXX
      min_cross_distance     = min_peak_peak_distance,
      min_cubicle_edge       = 5)
    psr = cgt.peak_search(parameters = peak_search_parameters,
      map = self.map_data).all(max_clusters = 99999999)
    self.sites_frac_water = psr.sites()
    self.ma.add("  total peaks found: %d"%self.sites_frac_water.size())
    if(self.debug): self._write_pdb_file(file_name_prefix="hoh_all_peaks")

  def _filter_by_distance(self):
    sel = mmtbx.utils.filter_water(
      sites_frac       = self.sites_frac_interaction,
      sites_frac_water = self.sites_frac_water,
      dist_min         = self.dist_min,
      dist_max         = self.dist_max,
      unit_cell        = self.unit_cell)
    self.sites_frac_water = self.sites_frac_water.select(sel)
    self.ma.add(
      "  distance (A), min: %4.2f max: %4.2f"%(self.dist_min, self.dist_max))
    self.ma.add("  peaks left: %d"%self.sites_frac_water.size())
    if(self.debug): self._write_pdb_file(file_name_prefix="hoh_dist")

  def _filter_by_sphericity(self):
    tmp = flex.vec3_double()
    for i, site_frac in enumerate(self.sites_frac_water):
      site_cart = self.unit_cell.orthogonalize(site_frac)
      # XXX make it so it is called once! XXX
      o = maptbx.sphericity_by_heuristics(
        map_data    = self.map_data,
        unit_cell   = self.unit_cell,
        center_cart = site_cart,
        radius      = 0.5)
      o2 = maptbx.sphericity_by_heuristics(
        map_data    = self.map_data,
        unit_cell   = self.unit_cell,
        center_cart = site_cart,
        radius      = 1.0)
      mi,ma,me = o.rho
      if(self.debug):
        _debug_show_all_plots(
          map_data    = self.map_data,
          unit_cell   = self.unit_cell,
          center_cart = site_cart,
          radius      = 1.0,
          plot_number = str("%d_%4.2f_%4.2f"%(i, o.ccs[0],o2.ccs[0])))
      fl = (mi>0 and ma>0 and me>0 and o.ccs[0]>0.95 and o2.ccs[0]>0.90)
      if(fl):
        tmp.append(site_frac)
        #print i, "%6.3f %6.3f %6.3f"%mv.min_max_mean().as_tuple(),\
        #  "%6.3f %6.3f %6.3f"%ccs.min_max_mean().as_tuple(),\
        #  "%6.3f %6.3f %6.3f"%ccs2.min_max_mean().as_tuple()
      #  print(hoh_str%(i,i,sc[0],sc[1],sc[2]), file=of)
      #else:
      #  print(hoh_str%(i,i,sc[0],sc[1],sc[2]), file=of2)
      #  #print i, "%6.3f %6.3f %6.3f"%mv.min_max_mean().as_tuple(),\
      #  #  "%6.3f %6.3f %6.3f"%ccs.min_max_mean().as_tuple(),\
      #  #  "%6.3f %6.3f %6.3f"%ccs2.min_max_mean().as_tuple(), "<<< REJECTED"
    self.sites_frac_water = tmp
    self.ma.add("  peaks left: %d"%self.sites_frac_water.size())

  def _show_peak_profiles(self):
    for i, site_frac in enumerate(self.sites_frac_water):
      site_cart = self.unit_cell.orthogonalize(site_frac)
      o = maptbx.sphericity_by_heuristics(
        map_data    = self.map_data,
        unit_cell   = self.unit_cell,
        center_cart = site_cart,
        radius      = 0.5)
      o2 = maptbx.sphericity_by_heuristics(
        map_data    = self.map_data,
        unit_cell   = self.unit_cell,
        center_cart = site_cart,
        radius      = 1.0)
      _debug_show_all_plots(
        map_data    = self.map_data,
        unit_cell   = self.unit_cell,
        center_cart = site_cart,
        radius      = 0.5,
        plot_number = str("%d_%4.2f_%4.2f"%(i+1, o.ccs[0],o2.ccs[0])))

  def _append_to_model(self):
    mean_b = flex.mean(self.model.get_hierarchy().atoms().extract_b())
    self.ma.add("  new water B factors will be set to mean B: %8.3f"%mean_b)
    sp = crystal.special_position_settings(self.model.crystal_symmetry())
    scatterers = flex.xray_scatterer()
    for site_frac in self.sites_frac_water:
      sc = xray.scatterer(
        label="O", site=site_frac, u=adptbx.b_as_u(mean_b), occupancy=1.0)
      scatterers.append(sc)
    xrs_water = xray.structure(sp, scatterers)
    #
    chain_ids_taken = []
    for chain in self.model.get_hierarchy().chains():
      chain_ids_taken.append(chain.id)
    for solvent_chain in all_chain_ids():
      if(not solvent_chain in chain_ids_taken):
        break
    self.ma.add("  new water will have chain ID: '%s'"%solvent_chain)
    #
    self.model.add_solvent(
      solvent_xray_structure = xrs_water,
      atom_name    = "O",
      residue_name = "HOH",
      chain_id     = solvent_chain,
      refine_adp   = "isotropic")



from __future__ import absolute_import, division, print_function
#!/usr/bin/python
# remediator.py - version 1.61.110622 6/22/11
# Copyright 2007-2011, Jeffrey J. Headd and Robert Immormino

# revision 1.56 - JJH 070808 - added support for DU DNA base
#               - JJH 070808 - added compiled RE object for HN2 RES special case
#       - JJH 070815 - updated name of hash dictionary file
#       - JJH 070823 - added support for CNS Xplor and Coot RNA names
#       - JJH 070908 - added REMARK   4 comment addition
#               - JJH 070913 - added support for left-justified RNA/DNA old names
#       - JJH 070913 - added support for all left-justified residue names
#       - JJH 080328 - fixed REMARK   4 comment addition to work w/ PHENIX header info
# revision 1.57 - vbc 080620 - lines get output with at least 80 columns
#                              master_hash.txt not found bug fixed, must be in same directory
#                              as this script
# revision 1.58 - JJH 080630 - fixed --old functionality for DNA, this did not work previously
#                              and would sometimes incorrectly rename protein atoms
#               - JJH 080827 - fixed a bug in this version only by restricting the number of
#                              substitutions to 1 per line
# revision 1.59 - JJH 081015 - added a --dict flag, which allows user to input a custom
#                              dictionary for detecting non-standard atom names and converting
#                              them to version 3.0
# revision 1.60 - JJH 081120 - reorganized code into functions and cleaned up flag usage
# revision 1.61 - JJH 110622 - moved to iotbx

import sys
import os
import re
import libtbx.load_env

#{{{ get_summary
def get_summary():
  summary = """
  iotbx.pdb_remediator model.pdb [params.eff] [options ...]

Options:

  file_name     specify input pdb file
  output_file   specify output file name (defaults to stdout)
  version       3.2 (default) or 2.3
  dict          optionally supply custom definition file

Examples:

Convert version 2.3 file to version 3.2 naming:

  iotbx.pdb_remediator model.pdb > model_3.2.pdb
  iotbx.pdb_remediator file_name=model.pdb output_file=model_3.2.pdb_file

Convert version 3.2 file to version 2.3 naming:

  iotbx.pdb_remediator model.pdb version=2.3 > model_2.3.pdb_file
"""
  return summary
#}}}

#{{{ pre_screen_file
def pre_screen_file(filename, atom_exch, alt_atom_exch):
  count = 0
  res_count = 0
  pdb_file = open(filename)
  for line in pdb_file:
    line=line.rstrip()
    line=line.ljust(80)
    type_test = line[0:6]
    if type_test in ("ATOM  ", "HETATM", "TER   ", "ANISOU", "SIGATM", "SIGUIJ", "LINK  "):
      adjust_res = False
      #--make any left-justified residue names right-justified------------------
      if re.match(r'.{17}([a-zA-Z0-9])  ',line):
        line = re.sub(r'\A(.{17})(.)\s\s',r'\g<1>  \g<2>',line)
        adjust_res = True
      elif re.match(r'.{17}([a-zA-Z0-9][a-zA-Z0-9]) ',line):
        line = re.sub(r'\A(.{17})(..)\s',r'\g<1> \g<2>',line)
        adjust_res = True
      #-------------------------------------------------------------------------

      #--pre-screen for CNS Xplor RNA base names and Coot RNA base names--------
      if re.match(r'.{17}(GUA|ADE|CYT|THY|URI)',line):
        line = re.sub(r'\A(.{17})(.)..',r'\g<1>  \g<2>',line)
        adjust_res = True
      elif re.match(r'.{17}(OIP| Ar| Gr| Cr| Ur)',line):
        line = re.sub(r'\A(.{17}).(.).',r'\g<1>  \g<2>',line)
        adjust_res = True
      #-------------------------------------------------------------------------

      if adjust_res:
        res_count += 1
      entry = line[12:20]
      clean_entry = entry[0:4] + " " + entry[5:8]
      if clean_entry in atom_exch:
        if clean_entry in alt_atom_exch:
          pass
        else:
          count += 1
  return count, res_count
#}}}

#{{{ build_hash
#--Build Hash Table------------------------------------------------
def build_hash(remediated_out, custom_dict, user_dict):
  atom_exch = {}
  file_name = os.path.join(libtbx.env.dist_path("iotbx"), "pdb",
    "remediation", "remediation.dict")
  f = open(file_name, "rb")
  if remediated_out == True: #converting to remediated
    for line in f:
      line=line.rstrip()
      new, old = line.split(':')
      atom_exch[old] = new
    remark4 = "REMARK   4 REMEDIATOR VALIDATED PDB VERSION 3.2 COMPLIANT".ljust(80)
  else: #converting to old
    for line in f:
      new, old = line.split(':')
      atom_exch[new] = old
    remark4 = "REMARK   4 REMEDIATOR VALIDATED PDB VERSION 2.3 COMPLIANT".ljust(80)
  if custom_dict == True:
    user_f = open(user_dict)
    if remediated_out == True: #converting to remediated
      for line in user_f:
        line=line.rstrip()
        new, old = line.split(':')
        atom_exch[old] = new
    else:
      for line in user_f:
        line=line.rstrip()
        new, old = line.split(':')
        atom_exch[new] = old
    user_f.close()
  f.close()
  return atom_exch, remark4
#}}}
#------------------------------------------------------------------

#{{{ remediate
#----PDB routine---------------------------------------------------
def remediate(filename, atom_exch, remediated_out, remark4, f=None):
  if f == None:
    f = sys.stdout
  previous = None
  current = None
  print_line = ""
  remark_flag = False
  remark_block = False

  pdb_file = open(filename)

  aa_re = re.compile(
    ' HN2 (ALA|ARG|ASN|ASP|ASX|CSE|CYS|GLN|GLU|GLX|GLY|HIS|ILE|'+
    'LEU|LYS|MET|MSE|PHE|PRO|SER|THR|TRP|UNK|TYR|VAL)')

  for line in pdb_file:
    line=line.rstrip()
    line=line.ljust(80)
    type_test = line[0:6]
    if remark_flag == False:
      if type_test == "REMARK":
        if remark_block == False:
          remark_block = True
        if re.search(remark4,line):
          remark_flag = True
        elif re.match(r'REMARK...\D',line):
          print_line += line + "\n"
          continue
        elif re.match('REMARK   4 REMEDIATOR',line):
          continue
        elif int(line[6:10]) > 4:
          print_line += remark4 + "\n"
          remark_flag = True
        else:
          print_line += line + "\n"
          continue

    if type_test in ("ATOM  ", "HETATM", "TER   ", "ANISOU", "SIGATM", "SIGUIJ", "LINK  "):
      if remark_flag == False:
        print_line += remark4 + "\n"
        remark_flag = True
      #--make any left-justified residue names right-justified------------------
      if re.match(r'.{17}([a-zA-Z0-9])  ',line):
        line = re.sub(r'\A(.{17})(.)\s\s',r'\g<1>  \g<2>',line)
      elif re.match(r'.{17}([a-zA-Z0-9][a-zA-Z0-9]) ',line):
        line = re.sub(r'\A(.{17})(..)\s',r'\g<1> \g<2>',line)
      #-------------------------------------------------------------------------

      #--pre-screen for CNS Xplor RNA base names and Coot RNA base names--------
      if re.match(r'.{17}(GUA|ADE|CYT|THY|URI)',line):
        line = re.sub(r'\A(.{17})(.)..',r'\g<1>  \g<2>',line)
      elif re.match(r'.{17}(OIP| Ar| Gr| Cr| Ur)',line):
        line = re.sub(r'\A(.{17}).(.).',r'\g<1>  \g<2>',line)
      #-------------------------------------------------------------------------

      entry = line[12:20]
      previous = current
      current = line[18:26]
      clean_entry = entry[0:4] + " " + entry[5:8]
      if clean_entry in atom_exch:
        line = line.replace(clean_entry[0:4],atom_exch[clean_entry][0:4],1)
    elif (remark_flag == False) and (remark_block == True): #deal with non-remark lines stuck in the top before main record types
      print_line += remark4 + "\n"
      remark_flag = True
    if previous == None:
      previous = current
    if previous == current:
      print_line += line + "\n"
    elif previous != current:
      if re.search(r'^.{12}.\S..  .[ACTGIU]',print_line):
        if re.search(r'O2[\'|\*]   .',print_line) == None:
          DNA_base = previous[1]
          if remediated_out == True:
            print_line = re.sub(r'(?m)(^.{12}.\S..)   '+DNA_base+' ',r'\g<1>  D'+DNA_base+' ',print_line)
            print_line = re.sub(r'(?m)(^TER.{15}) '+DNA_base+' ',r'\g<1>D'+DNA_base+' ',print_line)
          else:
            print_line = re.sub(r'(?m)(^.{12}.\S..)  D'+DNA_base+' ',r'\g<1>   '+DNA_base+' ',print_line)
            print_line = re.sub(r'(?m)(^TER.{15})D'+DNA_base+' ',r'\g<1> '+DNA_base+' ',print_line)

      if remediated_out == False:
        m = aa_re.search(print_line)
        if m:
          res = m.group(1)
          if re.search('1H   '+res,print_line) or re.search('2H   '+res,print_line):
            print_line = re.sub(' HN2 '+res,'2H   '+res,print_line)
      print_line=print_line.rstrip("\n")
      print(print_line, file=f)
      print_line = line + "\n"
  pdb_file.close()

  if re.search(r'^.{12}.\S..  .[ACTGIU]',print_line):
    if re.search(r'O2[\'|\*]   .',print_line) == None:
      DNA_base = previous[1]
      if remediated_out == True:
        print_line = re.sub(r'(?m)(^.{12}.\S..)   '+DNA_base,r'\g<1>  D'+DNA_base,print_line)
        print_line = re.sub(r'(?m)(^TER.{15}) '+DNA_base+' ',r'\g<1>D'+DNA_base+' ',print_line)
      else:
        print_line = re.sub(r'(?m)(^.{12}.\S..)  D'+DNA_base,r'\g<1>   '+DNA_base,print_line)
        print_line = re.sub(r'(?m)(^TER.{15})D'+DNA_base+' ',r'\g<1> '+DNA_base+' ',print_line)

    if remediated_out == False:
      m = aa_re.search(print_line)
      if m:
        res = m.group(1)
        if re.search('1H   '+res,print_line) or re.search('2H   '+res,print_line):
          print_line = re.sub(' HN2 '+res,'2H   '+res,print_line)

  print_line=print_line.rstrip("\n")
  print(print_line, file=f)
#}}}

def remediator(params, log=None):
  if log == None:
    log = sys.stderr
  custom_dict = False
  remedidated_out = True
  user_dict = ""
  file_name = params.file_name
  if params.version == "3.2":
    remediated_out = True
  elif params.version == "2.3":
    remediated_out = False
  if params.dict != None:
    custom_dict = True
    user_dict = params.dict
  if params.output_file != None:
    f = open(params.output_file, "w")
  else:
    f = sys.stdout
  atom_exch, remark4 = build_hash(remediated_out,
                                  custom_dict,
                                  user_dict)
  if remediated_out:
    remediated_alt = False
  else:
    remediated_alt = True
  alt_atom_exch, remark4_dump = build_hash(remediated_alt,
                                           custom_dict,
                                           user_dict)
  count, res_count = \
    pre_screen_file(file_name, atom_exch, alt_atom_exch)
  if count > 0 or res_count > 0:
    remediate(params.file_name, atom_exch, remediated_out, remark4, f)
    if params.output_file != None:
      f.close()
  else:
    if remediated_out:
      print("All atoms conform to PDB v3.x standard  **skipping**", file=log)
    else:
      print("All atoms conform to PDB v2.3 standard  **skipping**", file=log)




# XXX depends on internet connectivity, so not run as part of standard tests

from __future__ import absolute_import, division, print_function

def exercise():
  from mmtbx.wwpdb import rcsb_web_services
  lysozyme = """KVFGRCELAAAMKRHGLDNYRGYSLGNWVCAAKFESNFNTQATNRNTDGSTDYGILQINSRWWCNDGRTPGSRNLCNIPCSALLSSDITASVNCAKKIVSDGNGMNAWVAWRNRCKGTDVQAWIRGCRL"""
  homologs = rcsb_web_services.sequence_search(lysozyme, d_max=2.0)
  assert (len(homologs) > 500)
  atp_binding = rcsb_web_services.chemical_id_search("ATP", protein_only=True)
  assert (len(atp_binding) > 650)
  report = rcsb_web_services.get_high_resolution_for_structures(atp_binding)
  assert (len(report) == len(atp_binding)) and (len(report[0]) == 2)
  ligand_info = rcsb_web_services.get_ligand_info_for_structures(['1mru'])
  assert (len(ligand_info) == 4)
  print("OK")

if (__name__ == "__main__"):
  exercise()
